@@ -50,7 +50,7 @@ class UserClassifier:
             "inactiveUserCount": 0,
             "remoteExcludeReasonCount": defaultdict(int),
                     
-            "activeUserRefs": set(st3_081sByUserRef),
+            "activeUserRefs": set(st3_081sByUserRef.keys()),
             
             "dodUserRef": "", # special machine 'proxy' (CAPRI, not av)
             "postmasterUserRef": "",
@@ -103,7 +103,7 @@ class UserClassifier:
             len(self.__userInfoByUserRef)
         ))
         
-        self.__result["postmasterUserRef"] = self.__enforcePostmaster() 
+        self.__result["postmasterUserRef"] = self.__findPostmaster() # really an enforce 
         self.__result["dodUserRef"] = self.__findDoDUser() # one off
                 
         for userRef in self.__userInfoByUserRef:
@@ -146,17 +146,11 @@ class UserClassifier:
     Expect:
     - NO ssn
     """
-    def __enforcePostmaster(self):
+    def __findPostmaster(self):
         postmasters = [userRef for userRef in self.__userInfoByUserRef if re.match(r'POSTMASTER \[', userRef)]
-        if len(postmasters) == 0:
-            raise Exception("No POSTMASTER")
         if len(postmasters) > 1:
-            warning = "> 1 User Named Postmaster: {}".format("/".join(postmasters))
-            print("** Warning PostMaster: {}".format(warning))
-        postmaster = [userRef for userRef in postmasters if re.search(r'200\-\.5\]', userRef)]
-        if len(postmaster) != 1:
-            raise Exception("POSTMASTER not 200-.5")
-        userRef = postmaster[0]
+            raise Exception("> 1 user has name POSTMASTER")
+        userRef = postmasters[0]
         if not re.search(r'\[200-\.5\]$', userRef):
             raise Exception("IEN not .5")
         if userRef not in self.__st3_081sByUserRef:
@@ -214,7 +208,7 @@ class UserClassifier:
             
         if len(candidatesByVisitedFromCount) == 0:
             raise Exception("Can't reduce DOD User - none!")            
-        maxCount = sorted(candidatesByVisitedFromCount, reverse=True)[0]
+        maxCount = sorted(candidatesByVisitedFromCount.keys(), reverse=True)[0]
         if len(candidatesByVisitedFromCount[maxCount]) > 1:
             raise Exception("Can't reduce DOD User - ambiguous!")
             
@@ -292,11 +286,8 @@ class UserClassifier:
             raise Exception("Expected proxy user {} to have access verify".format(userRef))
                         
         if sum(1 for prop in signOnType if re.match(r'remote', prop)):
-            # saw in BOI 531 and TERMINAL EMULATOR remote_app (may be overload use?)
-            warning = "Unexpected \"remote_...\" property in SO of Proxy User"
-            self.__result["warningsByUserRef"][userRef].append(warning)
-            print("** Warning Proxy: {} for {}".format(warning, userRef))
-                                    
+            raise Exception("Unexpected \"remote_...\" property in SO of Proxy User {}".format(userRef))
+                        
         # HMP SPOK only => need to show
         if userInfo["user_class"].split(" ")[0] != "CONNECTOR":
             warning = "Only CONNECTOR Proxies but {}".format(userInfo["user_class"].split(" ")[0])
@@ -400,7 +391,7 @@ class UserClassifier:
         
 
     PUREZEROSOCREATEDUSER_200_MAND_PROPS = set([
-        "name", "alias", "name_components", "signature_block_printed_name", "date_entered", "xus_logon_attempt_count", "xus_active_user", "secondary_menu_options", "timestamp", "visited_from"
+        "name", "alias", "name_components", "signature_block_printed_name", "date_entered", "creator", "xus_logon_attempt_count", "xus_active_user", "secondary_menu_options", "timestamp", "visited_from"
     ])    
     
     PUREZEROSOCREATEDUSER_200_OPT_PROPS = set([    
@@ -453,8 +444,6 @@ class UserClassifier:
     
         """
         Pure JLV Subset
-
-        Since 2020, JLV WEB SERVICES as Context (only one) in JLV created users
         """
         def __testPureJLVSubset(userInfo, signOnType):
         
@@ -475,9 +464,10 @@ class UserClassifier:
             # Do warning on IP Addresses
                 
             return True
-
-        if "creator" in userInfo:
-            self.__result["remoteExcludeReasonCount"]["NO_CREATOR"] += 1
+    
+        # Forcing all - key subsetting
+        if "is_created_by_0" not in userInfo:
+            self.__result["remoteExcludeReasonCount"]["0_CREATED"] += 1
             return False
             
         ALLOWED_PROPS = UserClassifier.PUREZEROSOCREATEDUSER_200_MAND_PROPS.union(UserClassifier.PUREZEROSOCREATEDUSER_200_OPT_PROPS)
@@ -490,9 +480,11 @@ class UserClassifier:
         # ... all seem to have LOA 1
         if len(UserClassifier.PUREZEROSOCREATEDUSER_200_MAND_PROPS - set(userInfo["hasProps"])):
             self.__result["remoteExcludeReasonCount"]["MAND_PROPS"] += 1
-            missingMands = (UserClassifier.PUREZEROSOCREATEDUSER_200_MAND_PROPS - set(userInfo["hasProps"])) - set(["secondary_menu_options", "timestamp", "alias"])
+            missingMands = (UserClassifier.PUREZEROSOCREATEDUSER_200_MAND_PROPS - set(userInfo["hasProps"])) - set(["secondary_menu_options", "timestamp"])
             if len(missingMands) != 0:
-                self.__result["warningsByUserRef"][userRef].append("At this point in Remote Check, only expected mandatory SMO/ TimeStamp/ Alias could be missing but {}".format("/".join(list(missingMands))))
+                raise Exception("At this point in Remote Check, only expected mandatory SMO and TimeStamp could be missing but {}".format(
+                                    "/".join(list(missingMands))
+                                ))
             return False
             
         if "ssn" not in userInfo and sum(1 for vf in userInfo["signed_on_froms"] if re.match(r'358\:', vf)) == 0:
