@@ -5,15 +5,15 @@ import sys
 import os
 import re
 import json
-from collections import defaultdict, Counter
+from collections import defaultdict, OrderedDict, Counter
 from datetime import datetime, date
+import numpy
 
-from fmqlutils.reporter.reportUtils import MarkdownTable, reportPercent, reportAbsAndPercent, muBVC
-from fmqlutils.typer.reduceTypeUtils import splitTypeDatas, checkDataPresent, singleValue, combineSubTypes, muBVCOfSTProp
+from fmqlutils.reporter.reportUtils import MarkdownTable, reportPercent, reportAbsAndPercent
 
-from ..webReportUtils import TOP_MD_TEMPL, SITE_DIR_TEMPL, ensureWebReportLocations, keyStats, flattenFrequencyDistribution, roundFloat, reduce200
-from .userClassifier import UserClassifier
-from .USER_CONSTANTS import SSNO_DEFINITIONS
+from fmqlutils.typer.reduceTypeUtils import splitTypeDatas, checkDataPresent, singleValue, combineSubTypes, muBVC, muBVCOfSTProp
+from ..webReportUtils import TOP_MD_TEMPL, SITE_DIR_TEMPL, keyStats, flattenFrequencyDistribution, roundFloat, reduce200
+from userClassifier import UserClassifier
 
 """
 Basic User Type Summary/Overview
@@ -22,56 +22,34 @@ Unlike the stopcode centric reports (off locations), these are used and institut
 etc.
 
 Requires:
-- 3.081 (Type Reduction by User: YR1 
+- 3.081 (Type Reduction by User: YR1 unless 'smallSOSet')
 - 200 (All/Indexed-reduce200)
 - ... Will move to require image logs too to round out
 
 TODO: classification is work in progress - remotes from non key institution, locals
 from non subordinate divisions etc.
 
-Graph It To Emulate:
-https://matplotlib.org/gallery/showcase/bachelors_degrees_by_gender.html#sphx-glr-gallery-showcase-bachelors-degrees-by-gender-py - any dimension for users over time ... types of user?
+Note: smallSOSet setup for case where only have small sample of SOs (ie/ take all)  
 """
-def webReportUser(stationNo, deidentify=False):
+def webReportUser(stationNo, deidentify=False, smallSOSet=False):
 
     print("Preparing to make user report for {}{} - loading data ...".format(stationNo, " [DEIDENTIFY]" if deidentify else ""))
 
     allThere, details = checkDataPresent(stationNo, [
-        # Additional settings/properties:
-        # - duration (in seconds) 
-        # - force count on 
-        #     ipv4_address, remote_station_id, remote_user_ien, duration
-        {"fileType": "3_081", "check": "YR1E"}
+        # Additional settings: force count on 
+        #     ipv4_address, remote_station_id, remote_user_ien
+        {"fileType": "3_081", "check": "YR1" if not smallSOSet else "TYPE"},
+        {"fileType": "200", "check": "ALL"} # Index
     ])
     if not allThere:
         raise Exception("Some required data is missing - {}".format(details))
 
     mu = TOP_MD_TEMPL.format("{} Users".format(stationNo), "Users")
 
-    userInfoByUserRef = reduce200(stationNo) 
- 
-    """
-    Consider: if need to break by device (ie/ segment off SSH etc? See sniff) and
-    how to round out DOD USER breakdown ... are there any other 'CAPRI anyone' users
-    that must be broken too ie/ not just one remote user => must break out for remote
-    analysis
-    """
-    print("Loading subtypes ...")
-    try:
-        type3_081, st3_081URLDs = splitTypeDatas(stationNo, "3_081", reductionLabel="YR1E", expectSubTypeProperties=["user", "level_of_assurance", "remote_app", "remote_200_user_ien", "workstation_label", "workstation_type", "remote_station_id", "device"])
-    except:
-        print("Can't split type data - exiting")
-    sts3_081ByUserRef = defaultdict(list)
-    for st in st3_081URLDs:
-        userRef = singleValue(st, "user")
-        sts3_081ByUserRef[userRef].append(st)
+    userInfoByUserRef = reduce200(stationNo)    
+    type3_081, st3_081s = splitTypeDatas(stationNo, "3_081", reductionLabel="" if smallSOSet else "YR1", expectSubTypeProperty="user")
+    st3_081ByUserRef = dict((singleValue(st, "user"), st) for st in st3_081s if "user" in st)
     
-    # BACKWARD COMPATIBLE ... TODO: make work with ULRD (and only recombine down below 
-    # where needed)
-    print("Recombining subtypes by user (backward compatible) ...")
-    sts3_081Us = combineSubTypes(st3_081URLDs, ["user"], forceCountProps=["ipv4_address", "remote_station_id", "remote_user_ien", "duration"])
-    print("... done")
-    st3_081ByUserRef = dict((singleValue(st, "user"), st) for st in sts3_081Us if "user" in st)
     userClassifier = UserClassifier(stationNo, userInfoByUserRef, type3_081, st3_081ByUserRef)
     print("Classifying Users ...")
     classification = userClassifier.classify()
@@ -94,7 +72,7 @@ Most Active Users are real people but there are "__Machine Users__" for speciali
         reportAbsAndPercent(len(classification["activeUserRefs"]), len(userInfoByUserRef)),
         overallFirstSODate, 
         overallLastSODate,
-        "the most recent year for which data is available"
+        "the period for which sign on logs are available" if smallSOSet else "the most recent year for which data is available"
     )  
     
     if deidentify:
@@ -113,7 +91,7 @@ Most Active Users are real people but there are "__Machine Users__" for speciali
     ])
     tbl.addRow([
         "DoD User",
-        re.search(r'\[200\-([^\]]+)', classification["dodUserRef"]).group(1),
+        muUserRef(classification["dodUserRef"]),
         reportAbsAndPercent(
             st3_081ByUserRef[classification["dodUserRef"]]["_total"],
             type3_081["_total"]
@@ -145,22 +123,24 @@ Most Active Users are real people but there are "__Machine Users__" for speciali
             type3_081["_total"]
         )
     ])
-    for usrCls, label in [
-        ("activeRemoteUserRefs", "Active __Remote Users__"),
-        ("activeLocalUserRefs", "Active __Local Users__"),
-        ("activeNotCategorizedUserRefs", "Active __Uncategorized__")
-    ]:
-        if len(classification[usrCls]):
-            tbl.addRow([
-                label, 
-                reportAbsAndPercent(len(classification[usrCls]), len(classification["activeUserRefs"])),
-                reportAbsAndPercent(
-                    sum(st3_081ByUserRef[userRef]["_total"] for userRef
- in classification[usrCls]),
-                    type3_081["_total"]
-                )
-            ])
-    
+    tbl.addRow([
+        "Active __Remote Users__", 
+        reportAbsAndPercent(len(classification["activeRemoteUserRefs"]), len(classification["activeUserRefs"])),
+        reportAbsAndPercent(
+            sum(st3_081ByUserRef[userRef]["_total"] for userRef
+ in classification["activeRemoteUserRefs"]),
+            type3_081["_total"]
+        )
+    ])
+    tbl.addRow([
+        "Active __Local Users__", 
+        reportAbsAndPercent(len(classification["activeLocalUserRefs"]), len(classification["activeUserRefs"])),
+        reportAbsAndPercent(
+            sum(st3_081ByUserRef[userRef]["_total"] for userRef
+ in classification["activeLocalUserRefs"]),
+            type3_081["_total"]
+        )
+    ])
     mu += "User type signon summary ...\n\n"
     mu += tbl.md() + "\n\n"
     
@@ -192,7 +172,7 @@ Most Active Users are real people but there are "__Machine Users__" for speciali
         userInfoByUserRef,
         st3_081ByUserRef,
         type3_081["_total"],
-        keyColsOnly=deidentify
+        keyColsOnly = deidentify
     )
 
     mu += webReportNonProxyMachineUsers(
@@ -227,17 +207,6 @@ Most Active Users are real people but there are "__Machine Users__" for speciali
         deidentify,
         keyColsOnly = deidentify
     )
-    
-    mu += webReportUnclassifiedUsers(
-        classification["activeNotCategorizedUserRefs"], 
-        classification["warningsByUserRef"],
-        len(classification["activeUserRefs"]), 
-        userInfoByUserRef, 
-        type3_081["_total"], 
-        st3_081ByUserRef,
-        deidentify,
-        keyColsOnly = deidentify
-    )    
 
     userSiteDir = SITE_DIR_TEMPL.format(stationNo)
     reportFile = userSiteDir + ("user.md" if not deidentify else "userDeId.md")
@@ -283,7 +252,7 @@ There are <span class='yellowIt'>{:,}</span> active __Proxy Machine Users__ (use
         st = st3_081ByUserRef[userRef]
         allSTs.append(st)
         pmoMU, smosMU, keysMU = muOptionsNKeys(userInfo)
-        if "duration" in st and "byValueCount" in st["duration"]: # TODO: remove and FORCE all to have duration once redo E
+        if "duration" in st:
             if st["_total"] > 1:
                 kstatsDur = keyStats(
                     flattenFrequencyDistribution(st["duration"]["byValueCount"])
@@ -356,13 +325,13 @@ Besides the _DoD User_, there are <span class='yellowIt'>{:,}</span> active __No
         pmoMU, smosMU, keysMU = muOptionsNKeys(userInfo)
         unexpectedMU = "" if userRef not in warningsByUserRef else "/ ".join(warningsByUserRef[userRef])
         if "remote_user_ien" in st:
-            if len(st["remote_user_ien"]["byValueCount"]) > 5:
-                remoteIENsMU = "_#{:,}_".format(len(st["remote_user_ien"]["byValueCount"]))
+            if len(st["remote_user_ien"]["byValueCount"].keys()) > 5:
+                remoteIENsMU = "_#{:,}_".format(len(st["remote_user_ien"]["byValueCount"].keys()))
             else:
-                remoteIENsMU = "/".join(st["remote_user_ien"]["byValueCount"])
+                remoteIENsMU = "/".join(st["remote_user_ien"]["byValueCount"].keys())
         else:
             remoteIENsMU = ""
-        if "duration" in st and "byValueCount" in st["duration"]: # TODO: remove
+        if "duration" in st:
             if st["_total"] > 1:
                 kstatsDur = keyStats(
                     flattenFrequencyDistribution(st["duration"]["byValueCount"])
@@ -425,7 +394,7 @@ One special non proxy, machine user, the __\"DoD User\"__ is used for JLV DoD ac
     
     tbl = MarkdownTable([":Property", ":Value"], includeNo=False)
     
-    tbl.addRow(["IEN", re.search(r'\[200\-([^\]]+)', userRef).group(1)])
+    tbl.addRow(["Name \[IEN\]", muUserRef(userRef)])
     tbl.addRow(["Date Entered", userInfo["date_entered"] if "date_entered" in userInfo else ""])
     tbl.addRow(["SSN", "__{}__".format(userInfo["ssn"])])
     pmoMU, smosMU, keysMU = muOptionsNKeys(userInfo)
@@ -437,7 +406,7 @@ One special non proxy, machine user, the __\"DoD User\"__ is used for JLV DoD ac
     noRemoteStationIds = len(st["remote_station_id"]["byValueCount"])
     tbl.addRow(["Station Ids", noRemoteStationIds])
     def topRemoteStationIds(st):
-        orderedTopCounts = {} 
+        orderedTopCounts = OrderedDict()
         for i, rsid in enumerate(sorted(st["remote_station_id"]["byValueCount"], key=lambda x: st["remote_station_id"]["byValueCount"][x], reverse=True), 1):
             if i > 5:
                 break
@@ -512,16 +481,13 @@ fmrepo/user/catag (move over)
 """
 def webReportRemoteUsers(activeRemoteUserRefs, warningsByUserRef, totalActiveUsers, userInfoByUserRef, totalSignOns, st3_081ByUserRef, remoteExcludeReasonCount, deidentify, keyColsOnly=False):
 
-    if len(activeRemoteUserRefs) == 0:
-        return ""
-
     remoteSignOnCountsByUserRef = dict((userRef, st3_081ByUserRef[userRef]["_total"]) for userRef in activeRemoteUserRefs)
     totalRemoteSignOns = sum(remoteSignOnCountsByUserRef[userRef] for userRef in remoteSignOnCountsByUserRef)
-    kstats = keyStats(list(remoteSignOnCountsByUserRef.values()))
+    kstats = keyStats(remoteSignOnCountsByUserRef.values())
 
     print("About to combine lot's of remote sts (takes time) ...")
     comboST = combineSubTypes([st3_081ByUserRef[userRef] for userRef in activeRemoteUserRefs], forceCountProps=["remote_station_id", "remote_app", "division"])[0]
-    print("... end combo'ing")
+    print ("... end combo'ing")
     
     mu = """## Remote Users
     
@@ -538,13 +504,13 @@ Remote users dominate in every VistA - <span class='yellowIt'>{}</span> - but ac
         len(remoteStationBVC),
         len(set(stId[0:3] for stId in remoteStationBVC))
     )    
-    tbl = MarkdownTable([":Remote Station Id", ":Count of Remote SignOns"], includeNo=False)
+    tbl = MarkdownTable([":Remote Station Id", "Count of Remote SignOns"])
     for i, stationId in enumerate(sorted(remoteStationBVC, key=lambda x: remoteStationBVC[x], reverse=True), 1):
         if i > 10:
             break
         stationMU = "__{}__".format(stationId)
-        if stationId in SSNO_DEFINITIONS:
-            stationMU = "__{} [{}]__".format(stationId, SSNO_DEFINITIONS[stationId])
+        if stationId == "459": # Tripler
+            stationMU = "__459 [Tripler/JLV Test]__"
         tbl.addRow([
             stationMU, 
             reportAbsAndPercent(remoteStationBVC[stationId], totalRemoteSignOns)
@@ -566,13 +532,13 @@ Remote users dominate in every VistA - <span class='yellowIt'>{}</span> - but ac
             return sid 
         if len(stationIdVC) == 1:
             return "{}:{}".format(
-                muTripler(list(stationIdVC)[0]), 
-                list(ienVC)[0]
+                muTripler(stationIdVC.keys()[0]), 
+                ienVC.keys()[0]
             )
         if len(ienVC) == 1:
             return "{}:{}".format(
-                "/".join([muTripler(sid) for sid in stationIdVC]), 
-                list(ienVC)[0]
+                "/".join([muTripler(sid) for sid in stationIdVC.keys()]), 
+                ienVC.keys()[0]
             )
         # TODO: match counts in both to assemble id
         return ""
@@ -592,7 +558,7 @@ Remote users dominate in every VistA - <span class='yellowIt'>{}</span> - but ac
             remote_app_count["UNIDENTIFIED"] = no_remote_app_count
         remoteAppMU = ", ".join(["{} ({})".format(remote_app.split(" [")[0], remote_app_count[remote_app]) for remote_app in sorted(remote_app_count, key=lambda x: remote_app_count[x], reverse=True)])
         wdCntr = expandByWeekDay(st)
-        if "duration" in st and "byValueCount" in st["duration"]:
+        if "duration" in st:
             if st["_total"] > 1:
                 kstatsDur = keyStats(
                     flattenFrequencyDistribution(st["duration"]["byValueCount"])
@@ -638,17 +604,14 @@ TODO:
 """
 def webReportLocalUsers(activeLocalUserRefs, warningsByUserRef, totalActiveUsers, userInfoByUserRef, totalSignOns, st3_081ByUserRef, deidentify, keyColsOnly=False):
 
-    if len(activeLocalUserRefs) == 0:
-        return ""
-
     totalLocalSignOns = sum(st3_081ByUserRef[userRef]["_total"] for userRef in activeLocalUserRefs)
     mu = """## Local Users
     
 There are <span class='yellowIt'>{}</span> active Local Users with <span class='yellowIt'>{}</span> signons.\n\n""".format(reportAbsAndPercent(len(activeLocalUserRefs), totalActiveUsers), reportAbsAndPercent(totalLocalSignOns, totalSignOns))
 
     comboST = combineSubTypes([st3_081ByUserRef[userRef] for userRef in activeLocalUserRefs], forceCountProps=["division"])[0]
-    mu += "Local users select <span class='yellowIt'>{:,}</span> divisions (SHOULD RESTRICT THIS SET) ...\n\n".format(len(comboST["division"]["byValueCount"]))
-    tbl = MarkdownTable([":Division", ":Count %"])
+    mu += "Local users select <span class='yellowIt'>{:,}</span> divisions ...\n\n".format(len(comboST["division"]["byValueCount"]))
+    tbl = MarkdownTable([":Division", "Count %"])
     for divisionRef in sorted(comboST["division"]["byValueCount"], key=lambda x: comboST["division"]["byValueCount"][x], reverse=True):
         tbl.addRow([
             "__{}__".format(re.sub(r'4\-', '', divisionRef)), 
@@ -657,7 +620,7 @@ There are <span class='yellowIt'>{}</span> active Local Users with <span class='
     mu += tbl.md() + "\n\n"
     
     mu += "And don't just sign on through device 0 ...\n\n"
-    tbl = MarkdownTable([":Device", "Count %"], includeNo=False)
+    tbl = MarkdownTable([":Device", "Count %"])
     for device in sorted(comboST["device"]["byValueCount"], key=lambda x: comboST["device"]["byValueCount"][x], reverse=True):
         tbl.addRow([
             "__{}__".format(device), 
@@ -667,11 +630,11 @@ There are <span class='yellowIt'>{}</span> active Local Users with <span class='
     
     # TODO: will break on these 
     mu += "And have multiple levels of assurance ...\n\n"
-    tbl = MarkdownTable([":LOA", "Count %"], includeNo=False)
+    tbl = MarkdownTable([":LOA", "Count %"])
     for loa in sorted(comboST["level_of_assurance"]["byValueCount"], key=lambda x: comboST["level_of_assurance"]["byValueCount"][x], reverse=True):
         tbl.addRow([
             "__{}__".format(device), 
-            reportAbsAndPercent(comboST["level_of_assurance"]["byValueCount"][device], comboST["_total"])
+            reportAbsAndPercent(comboST["level_of_assurance"]["byValueCount"][loa], comboST["_total"])
         ])
     mu += tbl.md() + "\n\n"
     
@@ -711,18 +674,6 @@ There are <span class='yellowIt'>{}</span> active Local Users with <span class='
     mu += tbl.md() + "\n\n"
         
     return mu
-    
-def webReportUnclassifiedUsers(activeNotCategorizedUserRefs, warningsByUserRef, totalActiveUsers, userInfoByUserRef, totalSignOns, st3_081ByUserRef, deidentify, keyColsOnly=False):
-
-    if len(activeNotCategorizedUserRefs) == 0:
-        return ""
-
-    totalUnclassfiedSignOns = sum(st3_081ByUserRef[userRef]["_total"] for userRef in activeNotCategorizedUserRefs)
-    mu = """## Unclassified Users
-    
-There are <span class='yellowIt'>{}</span> active unclassified Users with <span class='yellowIt'>{}</span> signons.\n\n""".format(reportAbsAndPercent(len(activeNotCategorizedUserRefs), totalActiveUsers), reportAbsAndPercent(totalUnclassfiedSignOns, totalSignOns))
-
-    return mu
        
 # TODO: change to calc length formally in case of gaps 
 def muSignOnPeriod(st):
@@ -730,7 +681,7 @@ def muSignOnPeriod(st):
         soPeriodMU = "EVERY MONTH"
     elif st["date_time"]["firstCreateDate"].split("T")[0] == st["date_time"]["lastCreateDate"].split("T")[0]:
         soPeriodMU = st["date_time"]["lastCreateDate"].split("T")[0]
-    elif re.search(r'\-', list(st["date_time"]["byValueCount"])[0]): # months
+    elif re.search(r'\-', st["date_time"]["byValueCount"].keys()[0]): # months
         soPeriodMU = "{} - {} ({})".format(st["date_time"]["firstCreateDate"].split("T")[0], st["date_time"]["lastCreateDate"].split("T")[0], len(st["date_time"]["byValueCount"]))
     else:      
         soPeriodMU = "{} - {}".format(st["date_time"]["firstCreateDate"].split("T")[0], st["date_time"]["lastCreateDate"].split("T")[0])
@@ -802,7 +753,7 @@ def expandByWeekDay(st, fullDays=False):
     createDatePropInfo = st[st["_createDateProp"]]
     dayCnter = countDaysInPeriod(createDatePropInfo["firstCreateDate"], createDatePropInfo["lastCreateDate"])
     weekdays=["Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday"] if fullDays else ["Mon", "Tues", "Wed", "Thurs", "Fri", "Sat", "Sun"]
-    cntr = {}
+    cntr = OrderedDict()
     byWeekDay = st["date_time"]["byWeekDay"]
     for wd in sorted(byWeekDay, key=lambda x: int(x)):
         if wd not in byWeekDay:
@@ -813,7 +764,6 @@ def expandByWeekDay(st, fullDays=False):
     return cntr
     
 def muSeconds(seconds):
-    seconds = int(seconds)
     (days, remainder) = divmod(seconds, 86400)
     (hours, remainder) = divmod(remainder, 3600)
     (minutes, seconds) = divmod(remainder, 60)
@@ -841,21 +791,29 @@ def muSeconds(seconds):
                
 def main():
 
-    assert sys.version_info >= (3, 6)
-
-    try:
-        stationNo = sys.argv[1]
-    except IndexError:
-        raise SystemExit("Usage _EXE_ STATIONNO [DEID]")
-
-    ensureWebReportLocations(stationNo)
+    assert(sys.version_info >= (2,7))
+    
+    if len(sys.argv) < 2:
+        print "need to specify station # ex/ 442 - exiting"
+        return
+        
+    stationNo = sys.argv[1]
+    
+    userSiteDir = SITE_DIR_TEMPL.format(stationNo)
+    if not os.path.isdir(userSiteDir):
+        raise Exception("Expect User Site to already exist with its basic contents")
     
     if len(sys.argv) == 3:
-        deidentify = True
+        deidentify = sys.argv[2]
     else:
         deidentify = False
+        
+    if len(sys.argv) == 4:
+        smallSOSet = True
+    else:
+        smallSOSet = False
     
-    webReportUser(stationNo, deidentify)
+    webReportUser(stationNo, deidentify, smallSOSet)
     
 if __name__ == "__main__":
     main()
